@@ -1,8 +1,9 @@
 from copy import deepcopy
 import unittest2
+import datetime
 from jsonobject.base import JsonObject, JsonArray
 from jsonobject import *
-from jsonobject.exceptions import DeleteNotAllowed, BadValueError, WrappingAttributeError
+from jsonobject.exceptions import DeleteNotAllowed, BadValueError
 
 
 class Features(JsonObject):
@@ -41,7 +42,7 @@ class Person(Document):
 
 class FamilyMember(Person):
     base_doc = 'Person'
-    brothers = ListProperty(lambda: FamilyMember)
+    brothers = ListProperty(lambda: Person)
 
 
 class JunkCD(JsonObject):
@@ -105,7 +106,7 @@ class JsonObjectTestCase(unittest2.TestCase):
         with self.assertRaises(AssertionError):
             danny.brothers = brothers
 
-        brothers = map(FamilyMember.wrap, brothers)
+        brothers = map(Person.wrap, brothers)
         danny.brothers = brothers
 
         self.assertEqual(danny.brothers, brothers)
@@ -130,16 +131,6 @@ class JsonObjectTestCase(unittest2.TestCase):
         self.assertEqual(danny.favorite_numbers, numbers)
         self.assertEqual(danny.to_json()['favorite_numbers'], numbers)
 
-    def test_bad_wrap(self):
-        for error_type in (AttributeError, WrappingAttributeError):
-            with self.assertRaises(error_type) as cm:
-                Person.wrap({'full_name': 'Danny Roberts'})
-            self.assertEqual(
-                unicode(cm.exception),
-                "can't set attribute corresponding to "
-                "'full_name' on a <class 'test.tests.Person'> "
-                "while wrapping {'full_name': 'Danny Roberts'}"
-            )
     def test_pickle(self):
         import pickle
         f1 = FamilyMember.wrap(self._danny_data())
@@ -310,14 +301,13 @@ class JsonObjectTestCase(unittest2.TestCase):
 
     def test_dynamic_dict_property(self):
         "dates copied from couchdbkit"
-        import datetime
 
         class Foo(JsonObject):
             my_datetime = DateTimeProperty()
             my_dict = DictProperty()
         foo = Foo()
         full_datetime = datetime.datetime(2009, 5, 10, 21, 19, 21, 127380)
-        normalized_datetime = datetime.datetime(2009, 5, 10, 21, 19, 21)
+        normalized_datetime = datetime.datetime(2009, 5, 10, 21, 19, 21, 127380)
 
         foo.my_datetime = full_datetime
         self.assertEqual(foo.my_datetime, normalized_datetime)
@@ -358,21 +348,6 @@ class JsonObjectTestCase(unittest2.TestCase):
             Foo.wrap({'bar': []})
         Foo.wrap({'bar': {'baz': {'string': ''}}})
 
-    def test_long(self):
-        class Dummy(JsonObject):
-            i = IntegerProperty()
-            l = ListProperty(int)
-            l2 = ListProperty(IntegerProperty)
-        d = Dummy()
-        longint = 2 ** 63
-        self.assertIsInstance(longint, long)
-        d.i = longint
-        self.assertEqual(d.i, longint)
-        d.l = [longint]
-        self.assertEqual(d.l, [longint])
-        d.l2 = [longint]
-        self.assertEqual(d.l2, [longint])
-
 
 class LazyValidationTest(unittest2.TestCase):
 
@@ -446,7 +421,6 @@ class LazyValidationTest(unittest2.TestCase):
 
 class PropertyTestCase(unittest2.TestCase):
     def test_date(self):
-        import datetime
         p = DateProperty()
         for string, date in [('1988-07-07', datetime.date(1988, 7, 7))]:
             self.assertEqual(p.wrap(string), date)
@@ -457,7 +431,6 @@ class PropertyTestCase(unittest2.TestCase):
             p.wrap('2000-01-01T00:00:00Z')
 
     def test_datetime(self):
-        import datetime
         p = DateTimeProperty()
         for string, dt in [('2011-01-18T12:38:09Z', datetime.datetime(2011, 1, 18, 12, 38, 9))]:
             self.assertEqual(p.wrap(string), dt)
@@ -468,7 +441,6 @@ class PropertyTestCase(unittest2.TestCase):
             p.wrap('1988-07-07')
 
     def test_time(self):
-        import datetime
         p = TimeProperty()
         for string, time in [('12:38:09', datetime.time(12, 38, 9))]:
             self.assertEqual(p.wrap(string), time)
@@ -594,9 +566,7 @@ class PropertyTestCase(unittest2.TestCase):
         with self.assertRaises(AttributeError):
             foo.hello
 
-
 class DynamicConversionTestCase(unittest2.TestCase):
-    import datetime
 
     class Foo(JsonObject):
         pass
@@ -670,6 +640,49 @@ class DynamicConversionTestCase(unittest2.TestCase):
         self.assertEqual(foo.to_json(), {'my_list': ['bar']})
 
 
+class DateTimeTest(unittest2.TestCase):
+
+    def setUp(self):
+        class Foo(JsonObject):
+            dt = DateTimeProperty()
+            t = TimeProperty()
+            stuff = DictProperty()
+        self.Foo = Foo
+
+    def test_6_digit(self):
+        foo = self.Foo({'dt': '2013-10-30T12:34:56.789012Z'})
+        self.assertEqual(foo.dt, datetime.datetime(2013, 10, 30, 12, 34, 56, 789012))
+        self.assertEqual(foo.to_json()['dt'], '2013-10-30T12:34:56.789012Z')
+
+    def test_3_digit(self):
+        foo = self.Foo({'dt': '2013-10-30T12:34:56.789Z'})
+        self.assertEqual(foo.dt, datetime.datetime(2013, 10, 30, 12, 34, 56, 789000))
+        self.assertEqual(foo.to_json()['dt'], '2013-10-30T12:34:56.789Z')
+
+    def test_0_digit(self):
+        foo = self.Foo({'dt': '2013-10-30T12:34:56Z'})
+        self.assertEqual(foo.dt, datetime.datetime(2013, 10, 30, 12, 34, 56, 0))
+        self.assertEqual(foo.to_json()['dt'], '2013-10-30T12:34:56Z')
+
+    def test_6_digit_dynamic(self):
+        foo = self.Foo({'dynamic': '2013-10-30T12:34:56.789012Z'})
+        self.assertEqual(foo.dynamic, datetime.datetime(2013, 10, 30, 12, 34, 56, 789012))
+        self.assertEqual(foo.to_json()['dynamic'], '2013-10-30T12:34:56.789012Z')
+
+    def test_timezone(self):
+        """
+        timezone information in a wrapped doc should stay but is not interpreted
+
+        There's currently no way to get a timezone into a DateTimeProperty
+        other than wrapping a object that already has it
+
+        """
+
+        foo = self.Foo({'dt': '2013-10-30T12:34:56.789+02'})
+        self.assertEqual(foo.dt, datetime.datetime(2013, 10, 30, 12, 34, 56, 789000))
+        self.assertEqual(foo.to_json()['dt'], '2013-10-30T12:34:56.789+02')
+
+
 class User(JsonObject):
     username = StringProperty()
     name = StringProperty()
@@ -680,7 +693,6 @@ class User(JsonObject):
 
 class TestReadmeExamples(unittest2.TestCase):
     def test(self):
-        import datetime
         user1 = User(
             name='John Doe',
             username='jdoe',
